@@ -369,6 +369,7 @@ show_connection (NMConnection *data, gpointer user_data)
 	NmCli *nmc = (NmCli *) user_data;
 	NMSettingConnection *s_con;
 	guint64 timestamp;
+	time_t timestamp_real;
 	char *timestamp_str;
 	char timestamp_real_str[64];
 
@@ -377,7 +378,8 @@ show_connection (NMConnection *data, gpointer user_data)
 		/* Obtain field values */
 		timestamp = nm_setting_connection_get_timestamp (s_con);
 		timestamp_str = g_strdup_printf ("%" G_GUINT64_FORMAT, timestamp);
-		strftime (timestamp_real_str, sizeof (timestamp_real_str), "%c", localtime ((time_t *) &timestamp));
+		timestamp_real = timestamp;
+		strftime (timestamp_real_str, sizeof (timestamp_real_str), "%c", localtime (&timestamp_real));
 
 		nmc->allowed_fields[0].value = nm_setting_connection_get_id (s_con);
 		nmc->allowed_fields[1].value = nm_setting_connection_get_uuid (s_con);
