@@ -22,17 +22,19 @@
 #define _GNU_SOURCE
 #include <dlfcn.h>
 #include <syslog.h>
-#include <execinfo.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
-#include <execinfo.h>
 #include <strings.h>
 #include <string.h>
 
+#ifdef ENABLE_CRASHTRACE
+#include <execinfo.h>
+#endif
+
 #include <glib/gi18n.h>
 
 #include "nm-logging.h"
@@ -209,12 +211,19 @@ nm_logging_domains_to_string (void)
 	return g_string_free (str, FALSE);
 }
 
-void _nm_log (const char *loc,
-              const char *func,
-              guint32 domain,
-              guint32 level,
-              const char *fmt,
-              ...)
+gboolean
+nm_logging_level_enabled (guint32 level)
+{
+	return !!(log_level & level);
+}
+
+void
+_nm_log (const char *loc,
+         const char *func,
+         guint32 domain,
+         guint32 level,
+         const char *fmt,
+         ...)
 {
 	va_list args;
 	char *msg;
@@ -229,14 +238,14 @@ void _nm_log (const char *loc,
 
 	if ((log_level & LOGL_DEBUG) && (level == LOGL_DEBUG)) {
 		g_get_current_time (&tv);
-		syslog (LOG_INFO, "<debug> [%ld.%ld] [%s] %s(): %s\n", tv.tv_sec, tv.tv_usec, loc, func, msg);
+		syslog (LOG_INFO, "<debug> [%ld.%ld] [%s] %s(): %s", tv.tv_sec, tv.tv_usec, loc, func, msg);
 	} else if ((log_level & LOGL_INFO) && (level == LOGL_INFO))
-		syslog (LOG_INFO, "<info> %s\n", msg);
+		syslog (LOG_INFO, "<info> %s", msg);
 	else if ((log_level & LOGL_WARN) && (level == LOGL_WARN))
-		syslog (LOG_WARNING, "<warn> %s\n", msg);
+		syslog (LOG_WARNING, "<warn> %s", msg);
 	else if ((log_level & LOGL_ERR) && (level == LOGL_ERR)) {
 		g_get_current_time (&tv);
-		syslog (LOG_ERR, "<error> [%ld.%ld] [%s] %s(): %s\n", tv.tv_sec, tv.tv_usec, loc, func, msg);
+		syslog (LOG_ERR, "<error> [%ld.%ld] [%s] %s(): %s", tv.tv_sec, tv.tv_usec, loc, func, msg);
 	}
 	g_free (msg);
 }
@@ -246,6 +255,7 @@ void _nm_log (const char *loc,
 static void
 fallback_get_backtrace (void)
 {
+#ifdef ENABLE_CRASHTRACE
 	void *frames[64];
 	Dl_info info;
 	size_t size;
@@ -273,6 +283,7 @@ fallback_get_backtrace (void)
 		}
 	}
 	syslog (LOG_CRIT, "******************* END **********************************");
+#endif /* ENABLE_CRASHTRACE */
 }
 
 
